import logging

from .history_monitor import BashMonitor
from .terminado_mini_server import TerminadoMiniServer

LOG = logging.getLogger(__name__)


class TerminalHandler:
    keys = ['shell']
    """
    Event handler responsible for managing terminal sessions for frontend xterm
    sessions to connect to. You need to instanciate this in order for frontend
    terminals to work.

    This EventHandler accepts messages that have a data['command'] key specifying
    a command to be executed.
    The API of each command is documented in their respective handler.
    """
    def __init__(self, *, port, user, workind_directory, histfile):
        """
        :param key: key this EventHandler listens to
        :param monitor: tfw.components.HistoryMonitor instance to read command history from
        """
        self.server_connector = None
        self._histfile = histfile
        self._historymonitor = None
        bash_as_user_cmd = ['sudo', '-u', user, 'bash']

        self.terminado_server = TerminadoMiniServer(
            '/terminal',
            port,
            workind_directory,
            bash_as_user_cmd
        )

        self.commands = {
            'write': self.write,
            'read': self.read
        }

        self.terminado_server.listen()

    def start(self):
        self._historymonitor = BashMonitor(self.server_connector, self._histfile)
        self._historymonitor.start()

    @property
    def historymonitor(self):
        return self._historymonitor

    def handle_event(self, message, _):
        try:
            data = message['data']
            message['data'] = self.commands[data['command']](data)
        except KeyError:
            LOG.error('IGNORING MESSAGE: Invalid message received: %s', message)

    def write(self, data):
        """
        Writes a string to the terminal session (on the pty level).
        Useful for pre-typing and executing commands for the user.

        :param data: TFW message data containing 'value'
                     (command to be written to the pty)
        """
        self.terminado_server.pty.write(data['value'])
        return data

    def read(self, data):
        """
        Reads the history of commands executed.

        :param data: TFW message data containing 'count'
                     (the number of history elements to return)
        :return dict: message with list of commands in data['history']
        """
        data['count'] = int(data.get('count', 1))
        if self.historymonitor:
            data['history'] = self.historymonitor.history[-data['count']:]
        return data

    def cleanup(self):
        self.terminado_server.stop()
        self.historymonitor.stop()
